# C3.js Dataviz Adapter

C3.js is a beautiful charting library build on D3.js. This SDK includes an adapter that makes it possible to visualize your data with C3.js by including the dependencies shown below and making a few quick modifications to a chart's configuration.

There are also several examples that use C3.js on [jsFiddle](http://jsfiddle.net/user/keen/fiddles/).

### Installation

```html
<!-- Load c3.css -->
<link href="https://cdnjs.cloudflare.com/ajax/libs/c3/0.4.10/c3.min.css" rel="stylesheet" type="text/css">

<!-- Load d3.js and c3.js -->
<script src="https://cdnjs.cloudflare.com/ajax/libs/d3/3.5.6/d3.min.js" charset="utf-8"></script>
<script src="https://cdnjs.cloudflare.com/ajax/libs/c3/0.4.10/c3.min.js"></script>

<!-- Load keen.js -->
<script src="http://d26b395fwzu5fz.cloudfront.net/3.3.0/keen.min.js"></script>
```

### Configuration

Set the `libary` to "c3" and select a `chartType` from the list below.

```javascript
// via client.draw
client.draw(query, document.getElementById('my-chart'), {
  library: 'c3',
  chartType: 'area'
});

// via Keen.Dataviz()
var chart = new Keen.Dataviz()
  .el(document.getElementById('my-chart'))
  .library('c3')
  .chartType('area')
  .prepare();
```

**Available `chartType` values:**

* [area](http://c3js.org/samples/chart_area.html)
* [bar](http://c3js.org/samples/chart_bar.html)
* [line](http://c3js.org/samples/simple_multiple.html)
* [gauge](http://c3js.org/samples/chart_gauge.html)
* [donut](http://c3js.org/samples/chart_donut.html)
* [pie](http://c3js.org/samples/chart_pie.html)
* [spline](http://c3js.org/samples/chart_spline.html)
* [step](http://c3js.org/samples/chart_step.html)
* [area-spline](http://c3js.org/samples/chart_area_stacked.html)
* [area-step](http://c3js.org/samples/chart_step.html)


### Date formatting

Date formatting is possible by passing either a string or function to `dateFormat`. In either case, this value will be set as the `axis.x.tick.format` configuration property. As a built-in feature of C3.js, functions are used as an iterator, receiving the date for each interval in milliseconds.

```javascript
// Client shorthand
client.draw(query, element, {
  // string:
  dateFormat: '%Y-%m',
  // or function:
  dateFormat: function(ms){
    var date = new Date(ms);
    return date.getFullYear();
  }
});

// Dataviz instance
var chart = new Keen.Dataviz()
  .el(element)
  // string:
  .dateFormat('%Y-%m')
  // or function:
  .dateFormat(function(ms){
    var date = new Date(ms);
    return date.getFullYear();
  })
  .render();
```


### Customization

The `chartOptions` config object can be used to pass custom options into the chart prior to rendering. Learn more about C3.js configuration options [here](http://c3js.org/examples.html).

```javascript
// via client.draw
client.draw(query, document.getElementById('my-chart'), {
  library: 'c3',
  chartType: 'area',
  chartOptions: {
    axis: {
      x: {
        type: 'timeseries',
        tick: {
          count: 6,
          format: '%Y-%m-%d'
        }
      },
      y: {
        label: "Total actions"
      },
      y2: {
        show: true,
        label: "Percent of total"
      }
    }
  }
});

// via Keen.Dataviz()
var chart = new Keen.Dataviz()
  .el(document.getElementById('my-chart'))
  .library('c3')
  .chartType('area')
  .chartOptions({
    axis: {
      x: {
        type: 'timeseries',
        tick: {
          count: 6,
          format: '%Y-%m-%d'
        }
      },
      y: {
        label: "Total actions"
      },
      y2: {
        show: true,
        label: "Percent of total"
      }
    }
  })
  .prepare();
```

[See this configuration in action in this jsFiddle](http://jsfiddle.net/keen/cyqLwg7a/)
